{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Data Processing" %}{% endblock %}

{% block main %}

<div class="cluster_templates">
    {{ cluster_templates_table.render }}
</div>

{% endblock %}
