{% load i18n sizeformat %}
<h3>{% trans "Service Configurations" %}</h3>
<div class="status row detail">
    <dl class="dl-horizontal">
        {% for service, config in template.node_configs.items %}
        <dt>{{ service }}</dt>
        <dd>
            {% if config %}
            <ul>
                {% for conf_name, conf_val in config.items %}
                    <li>
                        {% blocktrans %}{{ conf_name }}: {{ conf_val }}{% endblocktrans %}
                    </li>
                {% endfor %}
            </ul>
            {% else %}
            <h6>{% trans "No configurations" %}</h6>
            {% endif %}
        </dd>
        {% endfor %}
    </dl>

</div>
