{% extends "horizon/common/_modal_form.html" %}


{% load i18n %}

{% block form_id %}create_cluster_form{% endblock %}
{% block form_action %}{% url 'horizon:project:data_processing.clusters:create' %}{% endblock %}

{% block modal-header %}{% trans "Launch Cluster" %}{% endblock %}

{% block modal-body %}
    <div class="left">
        <fieldset>
            {% include "horizon/common/_form_fields.html" %}
        </fieldset>
    </div>
{% endblock %}

{% block modal-footer %}
    <input class="btn btn-primary pull-right" id="create_cluster_btn" type="submit" value="{% trans " Done" %}"/>
    <a href="{% url 'horizon:project:data_processing.clusters:index' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
