{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Choose plugin and version" %}{% endblock %}

{% block main %}
    {% include 'cluster_wizard/_plugin_select.html' %}
{% endblock %}
