{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Data Processing" %}{% endblock %}

{% block main %}
<style type="text/css">
    .job_origin_main, .job_origin_lib {
    width: 200px !important; }
    .job_binary_add_button, .job_binary_remove_button {
    width: 80px !important;
    margin-left: 5px; }
    .form-help-block {
    float: left;
    text-align: left;
    width: 300px; }
    .lib-input-div {
    float:left;
    width:320px; }
    .job-libs-display {
    float:left; }
    .actions_column {
    width: 210px !important; }
</style>
<div class="job_origins">
    {{ jobs_table.render }}
</div>
{% endblock %}
