{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Jobs" %}{% endblock %}

{% block main %}
<div class="row">
  <div class="col-sm-12">
    {{ tab_group.render }}
  </div>
</div>

<script type="text/javascript">
        addHorizonLoadEvent(function () {
            horizon.modals.addModalInitFunction(function (modal) {
                if (($(modal).find("#launch_job__selectpluginaction")).length > 0) {
                    var $navbar = $(modal).find(".nav-tabs");
                    if ($navbar.find("li").size() == 1) {
                        // hide tab bar for plugin/version modal wizard
                       $navbar.hide();
                    }

                    lower_limit = 0;
                    $(".count-field").change();
                    if ($(modal).find(".hidden_create_field").length > 0) {
                        var form = $(".hidden_create_field").closest("form");
                        var successful = false;
                        form.submit(function (e) {
                                var oldHref = $(".create_job_class")[0].href;
                                var plugin = $("#id_plugin_name option:selected").val();
                                var version = $("#id_" + plugin + "_version option:selected").val();
                                var job_id = $("#id_job_id").val();
                                form.find(".close").click();
                                $(".create_job_class")[0].href = "launch-job-new-cluster?" +
                                        "plugin_name=" + encodeURIComponent(plugin) +
                                        "&hadoop_version=" + encodeURIComponent(version) +
                                        "&job_id=" + encodeURIComponent(job_id);
                                $(".create_job_class").click();
                                $(".create_job_class")[0].href = oldHref;
                                return false;
                        });
                        $(".plugin_version_choice").closest(".form-group").hide();
                    }

                    //display version for selected plugin
                    $(document).on('change', '.plugin_name_choice', switch_versions);
                    function switch_versions() {
                        $(".plugin_version_choice").closest(".form-group").hide();
                        var plugin = $(this);
                        $("." + plugin.val() + "_version_choice").closest(".form-group").show();
                    }
                    $(".plugin_name_choice").change();
                }
            });
        });

        addExtraBinary = function (where_from) {
          var loc_type = where_from.previousSibling.name.contains("main") ? "main" : "lib";
          for(i=2; i <= $("[name=extra_locations]").val(); i++) {
            if (!$("[name=job_" + loc_type + "_" + i + "]").closest(".form-group").is(":visible")) {
                $("[name=job_" + loc_type + "_" + i + "]").closest(".form-group").show();
                break;
            }
          }
        };
</script>

{% endblock %}
