{% load i18n sizeformat %}
{% load url from future %}
<h3>{% trans "Cluster Template Configuration Overview" %}</h3>
<div class="status row detail">
  <dl class="well">
    {% if template.cluster_configs %}
      <dd>
        {% for service, service_conf in template.cluster_configs.items %}
          <h4>{{ service }}</h4>
          {% if service_conf %}
            <ul>
              {% for conf_name, conf_value in service_conf.items %}
                <li>{% blocktrans %}{{ conf_name }}: {{ conf_value }}{% endblocktrans %}</li>
              {% endfor %}
            </ul>
          {% else %}
            <h6>{% trans "No configurations" %}</h6>
          {% endif %}
        {% endfor %}
      </dd>
    {% else %}
      <dd>{% trans "Cluster configurations are not specified" %}</dd>
    {% endif %}
  </dl>
</div>
<h3>{% trans "Node Groups Configuration Overview" %}</h3>
<div class="status row detail">
  {% for node_group in template.node_groups %}
    <dl class="well">
      <h4>{% blocktrans with node_group_name=node_group.name %}Node Group Name: {{ node_group_name }}{% endblocktrans %}</h4>
      {% if node_group.node_configs %}
        <dd>
          {% for service, service_conf in node_group.node_configs.items %}
            <h4>{{ service }}</h4>
            {% if service_conf %}
              <ul>
                {% for conf_name, conf_value in service_conf.items %}
                  <li>{% blocktrans %}{{ conf_name }}: {{ conf_value }}{% endblocktrans %}</li>
                {% endfor %}
              </ul>
            {% else %}
              <h6>{% trans "No configurations" %}</h6>
            {% endif %}
          {% endfor %}
        </dd>
      {% else %}
        <dd>{% trans "Node group configurations are not specified" %}</dd>
      {% endif %}
    </dl>
  {% endfor %}
</div>
